@@ -21,6 +21,7 @@ from mistral import context as auth_context
 from mistral.db.v2 import api as db_api
 from mistral import exceptions as exc
 from mistral.services import workbooks as wb_service
+from mistral.services import workflows as wf_service
 from mistral.tests.unit.engine import base
 from mistral.workflow import states
 
@@ -233,7 +234,7 @@ class SubworkflowsTest(base.EngineTestCase):
         self.assertEqual(project_id, wf2_ex.project_id)
         self.assertIsNotNone(wf2_ex)
         self.assertDictEqual({}, wf2_ex.input)
-        self.assertDictEqual({'namespace': ''}, wf2_ex.params)
+        self.assertDictEqual({'namespace': '', 'env': {}}, wf2_ex.params)
 
         self._await(lambda: len(db_api.get_workflow_executions()) == 2, 0.5, 5)
 
@@ -367,14 +368,8 @@ class SubworkflowsTest(base.EngineTestCase):
         wf1_ex = self._assert_single_item(wf_execs, name='wb1.wf1')
         wf2_ex = self._assert_single_item(wf_execs, name='wb1.wf2')
 
-        expected_start_params = {
-            'task_name': 'task2',
-            'task_execution_id': wf1_ex.task_execution_id,
-            'env': env
-        }
-
         self.assertIsNotNone(wf1_ex.task_execution_id)
-        self.assertDictContainsSubset(expected_start_params, wf1_ex.params)
+        self.assertDictContainsSubset({}, wf1_ex.params)
 
         # Wait till workflow 'wf1' is completed.
         self.await_workflow_success(wf1_ex.id)
@@ -458,6 +453,49 @@ class SubworkflowsTest(base.EngineTestCase):
             wf2_ex = db_api.get_workflow_execution(wf2_ex.id)
             wf3_ex = db_api.get_workflow_execution(wf3_ex.id)
 
-        self.assertIsNone(wf1_ex.root_execution_id, None)
-        self.assertEqual(wf2_ex.root_execution_id, wf1_ex.id)
-        self.assertEqual(wf3_ex.root_execution_id, wf1_ex.id)
+            self.assertIsNone(wf1_ex.root_execution_id, None)
+            self.assertEqual(wf2_ex.root_execution_id, wf1_ex.id)
+            self.assertEqual(wf2_ex.root_execution, wf1_ex)
+            self.assertEqual(wf3_ex.root_execution_id, wf1_ex.id)
+            self.assertEqual(wf3_ex.root_execution, wf1_ex)
+
+    def test_cascade_delete(self):
+        wf_text = """
+        version: 2.0
+
+        wf:
+          tasks:
+            task1:
+              workflow: sub_wf1
+
+            task2:
+              workflow: sub_wf2
+
+        sub_wf1:
+          tasks:
+            task1:
+              action: std.noop
+
+        sub_wf2:
+          tasks:
+            task1:
+              action: std.noop
+        """
+
+        wf_service.create_workflows(wf_text)
+
+        wf_ex = self.engine.start_workflow('wf')
+
+        self.await_workflow_success(wf_ex.id)
+
+        self.assertEqual(3, len(db_api.get_workflow_executions()))
+        self.assertEqual(4, len(db_api.get_task_executions()))
+        self.assertEqual(2, len(db_api.get_action_executions()))
+
+        # Now delete the root workflow execution and make sure that
+        # all dependent objects are deleted as well.
+        db_api.delete_workflow_execution(wf_ex.id)
+
+        self.assertEqual(0, len(db_api.get_workflow_executions()))
+        self.assertEqual(0, len(db_api.get_task_executions()))
+        self.assertEqual(0, len(db_api.get_action_executions()))
