@@ -20,11 +20,21 @@ from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from stevedore import extension
 import yaml
+from yaml import representer
 import yaql
 
+from yaql.language import utils as yaql_utils
+
+from mistral.config import cfg
 from mistral.db.v2 import api as db_api
 from mistral import utils
 
+# TODO(rakhmerov): it's work around the bug in YAQL.
+# YAQL shouldn't expose internal types to custom functions.
+representer.SafeRepresenter.add_representer(
+    yaql_utils.FrozenDict,
+    representer.SafeRepresenter.represent_dict
+)
 
 LOG = logging.getLogger(__name__)
 ROOT_YAQL_CONTEXT = None
@@ -39,7 +49,11 @@ def get_yaql_context(data_context):
         _register_yaql_functions(ROOT_YAQL_CONTEXT)
 
     new_ctx = ROOT_YAQL_CONTEXT.create_child_context()
-    new_ctx['$'] = data_context
+
+    new_ctx['$'] = (
+        data_context if not cfg.CONF.yaql.convert_input_data
+        else yaql_utils.convert_input_data(data_context)
+    )
 
     if isinstance(data_context, dict):
         new_ctx['__env'] = data_context.get('__env')
@@ -167,7 +181,8 @@ def execution_(context):
         'input': wf_ex.input,
         'params': wf_ex.params,
         'created_at': wf_ex.created_at.isoformat(' '),
-        'updated_at': wf_ex.updated_at.isoformat(' ')
+        'updated_at': wf_ex.updated_at.isoformat(' '),
+        'root_execution_id': wf_ex.root_execution_id
     }
 
 
