@@ -144,7 +144,8 @@ class WorkflowController(object):
             return []
 
         cmds = [
-            commands.RunExistingTask(self.wf_ex, self.wf_spec, t_e, reset)
+            commands.RunExistingTask(self.wf_ex, self.wf_spec, t_e, reset,
+                                     rerun=True)
             for t_e in task_execs
         ]
 
@@ -229,6 +230,11 @@ class WorkflowController(object):
         """
         raise NotImplementedError
 
+    def may_complete_workflow(self, task_ex):
+        """Determines if the task execution may lead to workflow completion."""
+
+        return states.is_completed(task_ex.state)
+
     @abc.abstractmethod
     def _find_next_commands(self, task_ex):
         """Finds commands that should run next.
