@@ -169,8 +169,6 @@ class DirectWorkflowController(base.WorkflowController):
         ctx = {}
 
         for batch in self._find_end_task_executions_as_batches():
-            if not batch:
-                break
 
             for t_ex in batch:
                 ctx = utils.merge_dicts(
@@ -200,6 +198,7 @@ class DirectWorkflowController(base.WorkflowController):
     def all_errors_handled(self):
         for t_ex in lookup_utils.find_error_task_executions(self.wf_ex.id):
             ctx_view = data_flow.ContextView(
+                data_flow.get_current_task_dict(t_ex),
                 data_flow.evaluate_task_outbound_context(t_ex),
                 data_flow.get_workflow_environment_dict(self.wf_ex),
                 self.wf_ex.context,
@@ -232,9 +231,14 @@ class DirectWorkflowController(base.WorkflowController):
         )
 
         for batch in batches:
-            yield list(
-                filter(is_end_task, batch)
-            )
+            yield list(filter(is_end_task, batch))
+
+    def may_complete_workflow(self, task_ex):
+        res = super(DirectWorkflowController, self).may_complete_workflow(
+            task_ex
+        )
+
+        return res and not self._has_outbound_tasks(task_ex)
 
     def _has_outbound_tasks(self, task_ex):
         # In order to determine if there are outbound tasks we just need
@@ -314,14 +318,16 @@ class DirectWorkflowController(base.WorkflowController):
     @profiler.trace('direct-wf-controller-find-downstream-joins')
     def _find_indirectly_affected_created_joins(self, task_name, result=None,
                                                 visited_task_names=None):
-        visited_task_names = visited_task_names or set()
+        visited_task_names = (
+            set() if visited_task_names is None else visited_task_names
+        )
 
         if task_name in visited_task_names:
             return
 
         visited_task_names.add(task_name)
 
-        result = result or set()
+        result = set() if result is None else result
 
         def _process_clause(clause):
             for t_name, condition, params in clause:
